package jig.engine.j2d;

import java.awt.Color;

import jig.engine.GameFrame;
import jig.engine.RenderingContext;
import jig.engine.ShapeEngine;
import jig.engine.j2d.J2DGameFrame.J2DRenderingContext;
import jig.engine.util.Vector2D;

public final class J2DShapeEngine implements ShapeEngine {

	J2DGameFrame gameframe;
	
	public J2DShapeEngine(GameFrame j2dgf) {
		try {
			gameframe = (J2DGameFrame) j2dgf;
		} catch (ClassCastException cce) {
			throw new IllegalArgumentException("ShapeEngine must be associated with a GameFrame from the J2DResourceFactory");
		}
	}
	
	public void renderVector(RenderingContext rc, Color c, Vector2D origin, Vector2D rendered) {
		J2DRenderingContext j2drc = (J2DRenderingContext) rc;
			
		
		double fx = origin.getX() + rendered.getX();
		double fy = origin.getY() + rendered.getY(); 

		Color orig = j2drc.theG.getColor();
		j2drc.theG.setColor(c);

		j2drc.theG.drawLine((int) origin.getX(), (int) origin.getY(), (int) fx, (int) fy);
		j2drc.theG.fillOval((int) fx-2, (int) fy - 2, 5, 5);
		j2drc.theG.setColor(orig);
	}
	
	public void renderLine(RenderingContext rc, Color c, double sx, double sy, double ex, double ey) {
		J2DRenderingContext j2drc = (J2DRenderingContext) rc;

		Color orig = j2drc.theG.getColor();
		j2drc.theG.setColor(c);
		j2drc.theG.drawLine((int) sx, (int) sy, (int) ex, (int) ey);
		j2drc.theG.setColor(orig);

	}
	
	public void renderCircle(RenderingContext rc, Color c, Vector2D topLeft, double radius) {
		J2DRenderingContext j2drc = (J2DRenderingContext) rc;

		Color orig = j2drc.theG.getColor();
		j2drc.theG.setColor(c);
		j2drc.theG.drawOval((int) topLeft.getX(),(int) topLeft.getY(),(int) radius,(int) radius);
		j2drc.theG.setColor(orig);
		
	}

	public void renderRectangle(RenderingContext rc, Color c, Vector2D ulc, Vector2D wh) {
		J2DRenderingContext j2drc = (J2DRenderingContext) rc;

		Color orig = j2drc.theG.getColor();
		j2drc.theG.setColor(c);
		j2drc.theG.drawRect((int) ulc.getX(),(int) ulc.getY(),(int) wh.getX(),(int) wh.getY());
		j2drc.theG.setColor(orig);
		
	}

	public void renderFilledRectangle(RenderingContext rc, Color c, Vector2D ulc, Vector2D wh) {
		J2DRenderingContext j2drc = (J2DRenderingContext) rc;

		Color orig = j2drc.theG.getColor();
		j2drc.theG.setColor(c);
		j2drc.theG.fillRect((int) ulc.getX(),(int) ulc.getY(),(int) wh.getX(),(int) wh.getY());
		j2drc.theG.setColor(orig);
		
	}
	
}
