package jig.engine.physics.vpe;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.util.List;

import jig.engine.ImageResource;
import jig.engine.RenderingContext;
import jig.engine.ResourceFactory;
import jig.engine.util.Vector2D;


public abstract class VanillaPolygon extends VanillaSphere {
	
	protected List<ImageResource> maskedFrames;
	
	protected ConvexPolygon shape;
	
	boolean shapeInSync = false;
	
	public VanillaPolygon(final List<ImageResource> frameset) {
		super(frameset);
		
	}
	public VanillaPolygon(final String rsc) {
		super(rsc);
	}
	
	
	public void setBackingShape(ConvexPolygon shape) {
		this.shape = shape;
		Vector2D sp = shape.getPosition();
		// temporarily do this
		shape.setPosition(new Vector2D(0,0));
		radius = shape.radius;
		shape.offsetToRotation = new Vector2D(radius, radius);
		
		BufferedImage[] mf = new BufferedImage[frames.size()];
		Graphics2D mfg;
		
		int x, y, w, h;
		int clear;
		clear = new Color(0,0,0,0).getRGB();
		for(int b = 0, n = frames.size(); b < n; b++ ) {
			w = frames.get(b).getWidth();
			h = frames.get(b).getHeight();
			mf[b] = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
			mfg = mf[b].createGraphics();
			mfg.setColor(new Color(128,128,128));
			mfg.fillRect(0,0,w,h);
			frames.get(b).draw(mfg, AffineTransform.getTranslateInstance(0, 0));
			
			for (x = 0; x < w; x++) {
				for (y = 0; y < h; y++) {
					if (!shape.contains(x, y)) {
						mf[b].setRGB(x,y,clear);
					}
				}
			}
		}

		
		shape.setPosition(sp);
		ResourceFactory.getFactory().putFrames("temp", mf);
		maskedFrames = ResourceFactory.getFactory().getFrames("temp");
		ResourceFactory.getFactory().freeResource("temp");
	}

	
	public Vector2D minPenetration(final VanillaPolygon s) {
		syncShape();
		s.syncShape();
		return shape.minPenetration(s.shape, false);
	}
	public Vector2D minPenetration(final VanillaPolygon s, boolean v) {
		syncShape();
		s.syncShape();
		return shape.minPenetration(s.shape, v);
	}

	@Override
	public void setPosition(Vector2D v) {
		shapeInSync = false;
		position = v;
	}
	@Override
	public void setRotation(double r) {
		shapeInSync = false;
		rotation = r;
	}
	public ConvexPolygon getShape() {
		syncShape();
		return shape;
	}
	public int getNSides() {
		return shape.nCorners;
	}
	private void syncShape() {
		if (shapeInSync) return;
		
		shape.setPosition(position);
		shape.setRotation(rotation);
	}
	@Override
	public void render(RenderingContext rc) {
		if (!active) return;
		
		syncShape();

		Vector2D p = position;
		AffineTransform at = AffineTransform.getTranslateInstance(p.getX()
				+ radius, p.getY() + radius);

		at.rotate(rotation);
		at.translate(-radius + offset.getX(), -radius + offset.getY());
		
		maskedFrames.get(visibleFrame).render(rc, at);
	
	}
}
