package jig.misc.sat;

import jig.engine.physics.vpe.ConvexPolygon;
import jig.engine.util.Vector2D;

/**
 * An Abstract Factory interface for making polygons.
 *
 * @author Scott Wallace
 *
 */
public interface PolygonFactory {
	/**
	 * Creates a rectangular polygon.
	 * 
	 * @param origin
	 *            position of the upper left corner
	 * @param w
	 *            the rectangle's width
	 * @param h
	 *            the rectangle's height
	 * @return a new rectangular polygon
	 */
	ConvexPolygon createRectangle(Vector2D origin, double w, double h);

	/**
	 * Creates a regular n-sided polygon.
	 * 
	 * @param origin
	 *            position of polygon's center
	 * @param radius
	 *            the radius of the circle that circumscribes the polygon
	 * @param n
	 *            the number of sides/corners
	 * @return a new regular n-sided polygon
	 */
	ConvexPolygon createNGon(Vector2D origin, double radius, int n);

}
