package jig.misc.sat;

import java.util.ArrayList;
import java.util.Formatter;
import java.util.Iterator;
import java.util.LinkedList;

import javax.imageio.spi.ServiceRegistry;

import jig.engine.GameClock;
import jig.engine.physics.vpe.ConvexPolygon;
import jig.engine.util.Vector2D;

public class SATBenchmark {
	private PolygonFactory factory;
	private LinkedList<PolygonFactory> polygonProviders;

	private ArrayList<ConvexPolygon> polygons;
	private int RADIUS = 32;
	
	public SATBenchmark() {
		// look up service providers for: PolygonFactory and SATLayer
		polygonProviders = new LinkedList<PolygonFactory>();
		for (Iterator<PolygonFactory> f = ServiceRegistry
				.lookupProviders(PolygonFactory.class); f.hasNext();) {
			polygonProviders.add(f.next());
		}
		if (polygonProviders.size() == 0) {
			throw new IllegalStateException(
					"Couldn't find a valid PolygonFactory service...is there a META-INFO/services entry?");
		}
		polygons = new ArrayList<ConvexPolygon>(100);
		
		int[] Ns = {4,8,16,32,64,128};
		int[] Ss = {3,4,5,6};
		char[] Ways = { '2' };
		
		Formatter f = new Formatter(System.out);

		f.format("%-25s %-5s %-5s %-5s %10s %10s\n",
				"Factory Name", "Objs", "Sides", "Collis", "Time (ms)", "FrmRate");
		
		long start, end, time;
		long collisions;
		for (PolygonFactory pf : polygonProviders) {
			factory = pf;
			for (int s : Ss) {
				for (int n : Ns){
					createPolygons(n, s);
					for (char w : Ways) {
						positionPolygons(w);
						collisions = 0;
						start = System.nanoTime();
						
						// n^2 collision test
						for (int i = 0; i < n; i++) {
							for (int j = i+1; j < n; j++) {
								if (polygons.get(i).minPenetration(polygons.get(j), false) != null) {
									collisions++;
								}
							}
						}
						end = System.nanoTime();
						time = end-start;
						f.format("%-25s %-5d %-5d %-5d %10.3f %10d\n",
								factory.getClass().getSimpleName(), n, s, collisions,
								((double)time)/GameClock.NANOS_PER_MS, GameClock.frameRateFromNanos(time));
						
					}
				}
			}
			
		}

	}
	public static void main(String[] args) {
		new SATBenchmark();
		
	}
	final void createPolygons(int n, int sides) {
		polygons.clear();
		polygons.ensureCapacity(n);
		Vector2D position = new Vector2D(0,0);
		for(int i = 0; i < n; i++) {
			polygons.add(factory.createNGon(position, RADIUS, sides ));
		}
	}
	
	final void positionPolygons(char way) {
		int i;
		int n = polygons.size();
		Vector2D v;
		switch (way){
			case 'R':  // row alignment
				for (i = 0; i < n; i++) {
					v = new Vector2D(i * 3 * RADIUS, 0);
					polygons.get(i).setPosition(v);
				}
				break;
			case 'C': // column alignment
				for (i = 0; i < n; i++) {
					v = new Vector2D(0, i * 3 * RADIUS);
					polygons.get(i).setPosition(v);
				}
				break;

			case '\\': // diagonal down and right
				for (i = 0; i < n; i++) {
					v = new Vector2D(i * 3 * RADIUS, i * 3 * RADIUS);
					polygons.get(i).setPosition(v);
				}
				break;
			case '/': // diagonal down and left
				for (i = 0; i < n; i++) {
					v = new Vector2D(i * 3 * RADIUS, (n - i) * 3 * RADIUS);
					polygons.get(i).setPosition(v);
				}
				break;
			case '1': // one overlapping pair

				polygons.get(0).setPosition(new Vector2D(2 * RADIUS, 0));
				for (i = 1; i < n; i++) {
					v = new Vector2D(i * 3 * RADIUS, 0);
					polygons.get(i).setPosition(v);
				}
				break;
			case '2': // overlapping pairs (n collisions)
				int width = n/2;
				for (i = 0; i < n; i++) {
					v = new Vector2D((i%width) * 3 * RADIUS, (i/width) * RADIUS);
					polygons.get(i).setPosition(v);
				}
				break;
				
		}
	}
}
