package jig.misc.sat;

import jig.engine.ViewableLayer;

/**
 * A ViewableLayer designed for use with the SATTestEnvironment. 
 * 
 * The SATLayer provides a visualization service to the SATTestEnvironment.
 * 
 * New layers can be provided by jar files and will be automatically
 * loaded by the SATTestEnvironment without modification.
 * 
 * @author Scott Wallace
 *
 */
public interface SATLayer extends ViewableLayer {
	
	/**
	 * Attaches the layer to the current environment.
	 * @param env a reference to the current environment.
	 */
	void setEnvironment(SATTestEnvironment env);
	
	/**
	 * Forces a refresh of the data in the layer 
	 * (which may only be updated periodically).
	 * 
	 * @param verbose <code>true</code> if it is acceptable to print
	 * debugging information to stderr or stdout
	 */
	void refresh(boolean verbose);
	
	/**
	 * 
	 * @return <code>true</code> if this layer is meant to overlay the
	 * standard SAT screen.  If this layer is meant to serve a distinct 
	 * function that is independent of the standard SAT visualization this
	 * method should return false.  
	 * 
	 * If a non-overlay layer is visible, no other layers will be visible.
	 */
	boolean isOverlay();
}
